-- to do
-- everything

-- turbine client imports
import "Turbine";
import "Turbine.Gameplay";
import "Turbine.UI";
import "Turbine.UI.Lotro";
-- need localPlayer to test for session play
localPlayer=Turbine.Gameplay.LocalPlayer.GetInstance();
-- mmediately test for session play BEFORE importing any other files so that we can safely bail without loading any data and without any handlers
if string.sub(localPlayer:GetName(),1,1)=="~" then
	-- do not load if the character is a session play character
	Turbine.Shell.WriteLine("Session play characters are not supported by Custom Tips.");
	return
end

string.ltrim=function(str)
	return (string.gsub(str, "^%s*(.-)", "%1"))
end
string.rtrim=function(str)
	return (string.gsub(str, "^(.-)%s*$", "%1"))
end
string.trim=function(str)
	return (string.gsub(str, "^%s*(.-)%s*$", "%1"))
end

import "GaranStuff.CustomTips.Class";
import "GaranStuff.CustomTips.VindarPatch";
-- This is the Load/Save patch which accounts for European formats, written by Vindar
-- load resource strings - this MUST occur before any windows that use strings get loaded or there will be no defaults
import "GaranStuff.CustomTips.Strings";
import "GaranStuff.CustomTips.Table";
import "GaranStuff.CustomTips.DropDownList";
import "GaranStuff.CustomTips.ColorPicker";
import "GaranStuff.CustomTips.RadioButtonGroup";
import "GaranStuff.CustomTips.DebugWindow";

-- set the language and locale prior to loading the panels as some of them have language/locale specific functions
language=1;
euroFormat=(tonumber("1,000")==1);
if euroFormat then
	function euroNormalize(value)
		return tonumber((string.gsub(value,"%.",",")));
	end
else
	function euroNormalize(value)
		return tonumber((string.gsub(value,",",".")));
	end
end

locale = "en";
if Turbine.Shell.IsCommand("hilfe") then
	locale = "de";
	language=3;
elseif Turbine.Shell.IsCommand("aide") then
	locale = "fr";
	language=2;
end

currentCharacterName=string.gsub(localPlayer:GetName(),"~","_");

mainVisible=false;
hudVisible=true; -- assume hud is visible when loading plugins -- I hate this assumption, but Turbine has failed to provide a legitimate means of testing the hud state

Settings=PatchDataLoad(Turbine.DataScope.Character, "CustomTipsSettings");
if Settings==nil then
	Settings={}
end

if Settings.debug==nil then Settings.debug=false end;
if Settings.fontColor==nil then
	Settings.fontColor=Turbine.UI.Color(1,.9,.5)
else
	Settings.fontColor=Turbine.UI.Color(euroNormalize(Settings.fontColor.R),euroNormalize(Settings.fontColor.G),euroNormalize(Settings.fontColor.B))
end
if Settings.errorColor==nil then
	Settings.errorColor=Turbine.UI.Color(1,0,0)
else
	Settings.errorColor=Turbine.UI.Color(euroNormalize(Settings.errorColor.R),euroNormalize(Settings.errorColor.G),euroNormalize(Settings.errorColor.B))
end

if Settings.trimColor==nil then
	Settings.trimColor=Turbine.UI.Color(.4,.4,.5)
else
	Settings.trimColor=Turbine.UI.Color(euroNormalize(Settings.trimColor.R),euroNormalize(Settings.trimColor.G),euroNormalize(Settings.trimColor.B))
end
if Settings.backColor==nil then
	Settings.backColor=Turbine.UI.Color(.05,.05,.05)
else
	Settings.backColor=Turbine.UI.Color(euroNormalize(Settings.backColor.R),euroNormalize(Settings.backColor.G),euroNormalize(Settings.backColor.B))
end
if Settings.panelBackColor==nil then
	Settings.panelBackColor=Turbine.UI.Color(.25,.75,.45)
else
	Settings.panelBackColor=Turbine.UI.Color(euroNormalize(Settings.panelBackColor.R),euroNormalize(Settings.panelBackColor.G),euroNormalize(Settings.panelBackColor.B))
end
if Settings.headingsColor==nil then
	Settings.headingsColor=Turbine.UI.Color(1,1,1)
else
	Settings.headingsColor=Turbine.UI.Color(euroNormalize(Settings.headingsColor.R),euroNormalize(Settings.headingsColor.G),euroNormalize(Settings.headingsColor.B))
end
if Settings.listTextColor==nil then
	Settings.listTextColor=Turbine.UI.Color(1,1,1)
else
	Settings.listTextColor=Turbine.UI.Color(euroNormalize(Settings.listTextColor.R),euroNormalize(Settings.listTextColor.G),euroNormalize(Settings.listTextColor.B))
end
if Settings.fontFace==nil then Settings.fontFace=Turbine.UI.Lotro.Font.Verdana16 end

customTips=PatchDataLoad(Turbine.DataScope.Character, "CustomTips");
if customTips==nil then
	customTips={}
end

-- need to deal with decimal values
for k,v in ipairs(customTips) do
	v.Color=Turbine.UI.Color(euroNormalize(v.Color.R),euroNormalize(v.Color.G),euroNormalize(v.Color.B))
	v.Opacity=euroNormalize(v.Opacity)
end
-- This is the callback mechanism provided by Pengoros, slightly modified to guarantee uniqueness
function AddCallback(object, event, callback)
	if (object[event] == nil) then
		object[event] = callback;
	else
		if (type(object[event]) == "table") then
			local exists=false;
			local k,v;
			for k,v in ipairs(object[event]) do
				if v==callback then
					exists=true;
					break;
				end
			end
			if not exists then
				table.insert(object[event], callback);
			end
		else
			if object[event]~=callback then
				object[event] = {object[event], callback};
			end
		end
	end
	return callback;
end

-- safely remove a callback without clobbering any extras
function RemoveCallback(object, event, callback)
    if (object[event] == callback) then
        object[event] = nil;
    else
        if (type(object[event]) == "table") then
            local size = table.getn(object[event]);
            for i = 1, size do
                if (object[event][i] == callback) then
                    table.remove(object[event], i);
                    break;
                end
            end
        end
    end
end

function UnloadPlugin()
	SaveData();

	-- have to remove all shell commands before unloading or we get a crash to desktop :(
	if mainWindow~=nil then
		Turbine.Shell.RemoveCommand(mainWindow.shellCmd)
	end

	-- safely remove all callbacks
--	RemoveCallback(localPlayer,"BaseMaxPowerChanged",BaseMaxPowerChanged)

end
function SaveData()

	-- save the tooltip settings
	local tmpIndex;
	Settings.version=Plugins.CustomTips:GetVersion();
	Settings.MWTop=mainWindow:GetTop();
	Settings.MWLeft=mainWindow:GetLeft();

	PatchDataSave( Turbine.DataScope.Character, "CustomTipsSettings", Settings );
	PatchDataSave( Turbine.DataScope.Character, "CustomTips", customTips );
end

mainWindow = Turbine.UI.Lotro.Window();
mainWindow.Loaded=false;
-- set size/position

	local initHeight=460;
	local initWidth=445;
	if Settings.MWTop==nil then Settings.MWTop=(Turbine.UI.Display:GetHeight() - initHeight) / 2 end
	if Settings.MWLeft==nil then Settings.MWLeft=(Turbine.UI.Display:GetWidth() - initWidth) / 2 end

	if initHeight>Turbine.UI.Display:GetHeight() then initHeight=Turbine.UI.Display:GetHeight() end
	if initHeight+Settings.MWTop>Turbine.UI.Display:GetHeight() then Settings.MWTop=Turbine.UI.Display:GetHeight()-initHeight end
	if initWidth>Turbine.UI.Display:GetWidth() then initWidth=Turbine.UI.Display:GetWidth() end
	if initWidth+Settings.MWLeft>Turbine.UI.Display:GetWidth() then Settings.MWLeft=Turbine.UI.Display:GetWidth()-initWidth end

	mainWindow:SetPosition( Settings.MWLeft, Settings.MWTop );
	mainWindow:SetHeight(initHeight);
	mainWindow:SetWidth(initWidth);
	mainWindow:SetOpacity( 1 );
	mainWindow:SetVisible(false);

	mainWindow:SetText(Resource[language][2]);

	mainWindow.KeyDown=function(sender, args)
		if bShowKeyDown==true then
			Turbine.Shell.WriteLine("KeyDown Action="..args.Action..", .Alt="..tostring(args.Alt)..", .Control="..tostring(args.Control)..", .Shift="..tostring(args.Shift))
		end
		if ( args.Action == Turbine.UI.Lotro.Action.Escape ) then
			mainWindow:SetVisible( false );
		end
		if (args.Action==43) then -- toggle debug
			Settings.debug=not Settings.debug;
			
			if debugWindow==nil then
				debugWindow=DebugWindow();
			end
			debugWindow:SetVisible(Settings.debug);
		end
		if ( args.Action == 268435579 ) then -- reposition UI
		end

		if ( args.Action == 268435635 ) then -- toggle HUD
			hudVisible=not hudVisible;
			if hudVisible then
				mainWindow:SetVisible(mainVisible);
			else
				mainVisible=mainWindow:IsVisible();
				mainWindow:SetVisible(false);
			end
		end
	end

	mainWindow:SetWantsKeyEvents(true);
	mainWindow:SetWantsUpdates(true)

	mainWindow.shellCmd=Turbine.ShellCommand();
	local numberOfCommandsRegistered = Turbine.Shell.AddCommand("CustomTips",mainWindow.shellCmd);

	mainWindow.shellCmd.Execute = function(sender, cmd, args)
		if (string.lower(cmd) == "customtips") then
			if (string.lower(args)=="setup") then
				mainWindow:SetVisible(true);
			elseif (string.lower(args)=="hide") then
				mainWindow:SetVisible(false);
			elseif (string.lower(args)=="toggle") then
				mainWindow:SetVisible(not mainWindow:IsVisible());
			elseif (string.lower(args)=="debug on") then
				Settings.debug=true;
				if debugWindow==nil then
					debugWindow=DebugWindow();
				end
				debugWindow:SetVisible(true);
			elseif (string.lower(args)=="debug off") then
				Settings.debug=false;
				if debugWindow~=nil then
					debugWindow:SetVisible(false);
				end
			else
				Turbine.Shell.WriteLine("usage:");
				Turbine.Shell.WriteLine("  /CustomTips setup");
				Turbine.Shell.WriteLine("       Displays the Custom Tips main window.");
				Turbine.Shell.WriteLine("  /CustomTips hide");
				Turbine.Shell.WriteLine("       Hides the Custom Tips main window.");
				Turbine.Shell.WriteLine("  /Custom Tips toggle");
				Turbine.Shell.WriteLine("       Toggles the Custom Tips main window (shows it if it was hidden, hides it if it was showing).");
			end
		end
	end
	mainWindow.shellCmd.GetHelp = function(sender, cmd)
		return("usage:\n  /CustomTips setup\n       Displays the Custom Tips main window.\n  /CustomTips hide\n       Hides the Custom Tips main window.\n  /CustomTips toggle\n       Toggles the Custom Tips main window (shows it if it was hidden, hides it if it was showing).");
	end

	mainWindow.shellCmd.GetShortHelp = function(sender, cmd)
		return("Custom Tips - usage: /CustomTips setup | hide | toggle");
	end

	mainWindow.SetTrimColor=function()
	end
	mainWindow.SetBackColor=function()
	end
	mainWindow.SetPanelBackColor=function()
	end
	mainWindow.SetListTextColor=function()
	end
	mainWindow.SetFontColor=function()
	end
	mainWindow.SetHeadingsColor=function()
	end

-- mainWindow Interface
	mainWindow.SelCaption=Turbine.UI.Label();
	mainWindow.SelCaption:SetParent(mainWindow);
	mainWindow.SelCaption:SetPosition(5,45);
	mainWindow.SelCaption:SetSize(100,20);
	mainWindow.SelCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
	mainWindow.SelCaption:SetFont(Settings.fontFace);
	mainWindow.SelCaption:SetForeColor(Settings.fontColor);
	mainWindow.SelCaption:SetText(Resource[language][4]..":");
	mainWindow.SelList=DropDownList();
	mainWindow.SelList:SetParent(mainWindow);
	mainWindow.SelList:SetPosition(mainWindow.SelCaption:GetLeft()+mainWindow.SelCaption:GetWidth()+5,mainWindow.SelCaption:GetTop())
	mainWindow.SelList:SetSize(mainWindow:GetWidth()-mainWindow.SelList:GetLeft()-10,mainWindow.SelCaption:GetHeight());
	mainWindow.SelList:SetFont(Turbine.UI.Lotro.Font.Verdana16);
	mainWindow.SelList:SetCurrentBackColor(Settings.backColor);
	mainWindow.SelList:SetBackColor(Settings.backColor);
	mainWindow.SelList:SetBorderColor(Settings.trimColor);
	mainWindow.SelList:SetZOrder(100);
	mainWindow.SelList.SelectedIndexChanged=function()
		local newIndex=mainWindow.SelList:GetValue()
		if newIndex>0 then
			mainWindow.NameValue:SetText(customTips[newIndex].Name);
			mainWindow.LeftValue:SetText(customTips[newIndex].Left);
			mainWindow.TopValue:SetText(customTips[newIndex].Top);
			mainWindow.WidthValue:SetText(customTips[newIndex].Width);
			mainWindow.HeightValue:SetText(customTips[newIndex].Height);
			mainWindow.ColorValue:SetColor(customTips[newIndex].Color);
			mainWindow.OpacityValue:SetValue(customTips[newIndex].Opacity*100);
			mainWindow.IconQS:SetVisible(true);
			mainWindow.IconII:SetVisible(false);
			if customTips[newIndex].IconType~=nil then
if Settings.debug then
Turbine.Shell.WriteLine("IconType:"..tostring(customTips[newIndex].IconType))
Turbine.Shell.WriteLine("IconData:"..tostring(customTips[newIndex].IconData))
end
				local sc=Turbine.UI.Lotro.Shortcut(customTips[newIndex].IconType,0);
				sc:SetData(customTips[newIndex].IconData);
				mainWindow.IconQS:SetShortcut(sc);
				mainWindow.IconQS.Type=customTips[newIndex].IconType;
				mainWindow.IconQS.Data=customTips[newIndex].IconData;
				if customTips[newIndex].IconType==2 then
					mainWindow.IconQS:SetVisible(false);
					mainWindow.IconII:SetVisible(true);
					mainWindow.IconII:SetItemInfo(sc:GetItem():GetItemInfo())
--					mainWindow.IconII:SetWantsUpdates(true);
				end
			else
				mainWindow.IconQS:SetShortcut(nil);
			end
			mainWindow.TextValue:SetText(customTips[newIndex].Text);
		end
		mainWindow.DeleteButton:SetEnabled(newIndex>0);
		layoutWindow:Display();
	end
	mainWindow.Sep1=Turbine.UI.Control();
	mainWindow.Sep1:SetParent(mainWindow);
	mainWindow.Sep1:SetBackColor(Settings.trimColor);
	mainWindow.Sep1:SetSize(mainWindow:GetWidth()-10,1);
	mainWindow.Sep1:SetPosition(5,mainWindow.SelCaption:GetTop()+mainWindow.SelCaption:GetHeight()+2);

	mainWindow.NameCaption=Turbine.UI.Label();
	mainWindow.NameCaption:SetParent(mainWindow);
	mainWindow.NameCaption:SetPosition(5,mainWindow.Sep1:GetTop()+mainWindow.Sep1:GetHeight()+5);
	mainWindow.NameCaption:SetSize(mainWindow.SelCaption:GetWidth(),20);
	mainWindow.NameCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
	mainWindow.NameCaption:SetFont(Settings.fontFace);
	mainWindow.NameCaption:SetForeColor(Settings.fontColor);
	mainWindow.NameCaption:SetText(Resource[language][6]..":");
	mainWindow.NameValue=Turbine.UI.Lotro.TextBox();
	mainWindow.NameValue:SetParent(mainWindow);
	mainWindow.NameValue:SetSize(mainWindow.SelList:GetWidth(),20);
	mainWindow.NameValue:SetPosition(mainWindow.SelList:GetLeft(),mainWindow.NameCaption:GetTop());
	mainWindow.NameValue:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	mainWindow.NameValue:SetFont(Settings.fontFace);

	mainWindow.LeftCaption=Turbine.UI.Label();
	mainWindow.LeftCaption:SetParent(mainWindow);
	mainWindow.LeftCaption:SetPosition(5,mainWindow.NameCaption:GetTop()+mainWindow.NameCaption:GetHeight()+5);
	mainWindow.LeftCaption:SetSize(mainWindow.SelCaption:GetWidth(),20);
	mainWindow.LeftCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
	mainWindow.LeftCaption:SetFont(Settings.fontFace);
	mainWindow.LeftCaption:SetForeColor(Settings.fontColor);
	mainWindow.LeftCaption:SetText(Resource[language][7]..":");
	mainWindow.LeftValue=Turbine.UI.Lotro.TextBox();
	mainWindow.LeftValue:SetParent(mainWindow);
	mainWindow.LeftValue:SetSize(mainWindow.SelList:GetWidth(),20);
	mainWindow.LeftValue:SetPosition(mainWindow.SelList:GetLeft(),mainWindow.LeftCaption:GetTop());
	mainWindow.LeftValue:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	mainWindow.LeftValue:SetFont(Settings.fontFace);
	mainWindow.LeftValue:SetText("0");

	mainWindow.TopCaption=Turbine.UI.Label();
	mainWindow.TopCaption:SetParent(mainWindow);
	mainWindow.TopCaption:SetPosition(5,mainWindow.LeftCaption:GetTop()+mainWindow.LeftCaption:GetHeight()+5);
	mainWindow.TopCaption:SetSize(mainWindow.SelCaption:GetWidth(),20);
	mainWindow.TopCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
	mainWindow.TopCaption:SetFont(Settings.fontFace);
	mainWindow.TopCaption:SetForeColor(Settings.fontColor);
	mainWindow.TopCaption:SetText(Resource[language][8]..":");
	mainWindow.TopValue=Turbine.UI.Lotro.TextBox();
	mainWindow.TopValue:SetParent(mainWindow);
	mainWindow.TopValue:SetSize(mainWindow.SelList:GetWidth(),20);
	mainWindow.TopValue:SetPosition(mainWindow.SelList:GetLeft(),mainWindow.TopCaption:GetTop());
	mainWindow.TopValue:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	mainWindow.TopValue:SetFont(Settings.fontFace);
	mainWindow.TopValue:SetText("0");

	mainWindow.WidthCaption=Turbine.UI.Label();
	mainWindow.WidthCaption:SetParent(mainWindow);
	mainWindow.WidthCaption:SetPosition(5,mainWindow.TopCaption:GetTop()+mainWindow.TopCaption:GetHeight()+5);
	mainWindow.WidthCaption:SetSize(mainWindow.SelCaption:GetWidth(),20);
	mainWindow.WidthCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
	mainWindow.WidthCaption:SetFont(Settings.fontFace);
	mainWindow.WidthCaption:SetForeColor(Settings.fontColor);
	mainWindow.WidthCaption:SetText(Resource[language][9]..":");
	mainWindow.WidthValue=Turbine.UI.Lotro.TextBox();
	mainWindow.WidthValue:SetParent(mainWindow);
	mainWindow.WidthValue:SetSize(mainWindow.SelList:GetWidth(),20);
	mainWindow.WidthValue:SetPosition(mainWindow.SelList:GetLeft(),mainWindow.WidthCaption:GetTop());
	mainWindow.WidthValue:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	mainWindow.WidthValue:SetFont(Settings.fontFace);
	mainWindow.WidthValue:SetText("32");

	mainWindow.HeightCaption=Turbine.UI.Label();
	mainWindow.HeightCaption:SetParent(mainWindow);
	mainWindow.HeightCaption:SetPosition(5,mainWindow.WidthCaption:GetTop()+mainWindow.WidthCaption:GetHeight()+5);
	mainWindow.HeightCaption:SetSize(mainWindow.SelCaption:GetWidth(),20);
	mainWindow.HeightCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
	mainWindow.HeightCaption:SetFont(Settings.fontFace);
	mainWindow.HeightCaption:SetForeColor(Settings.fontColor);
	mainWindow.HeightCaption:SetText(Resource[language][10]..":");
	mainWindow.HeightValue=Turbine.UI.Lotro.TextBox();
	mainWindow.HeightValue:SetParent(mainWindow);
	mainWindow.HeightValue:SetSize(mainWindow.SelList:GetWidth(),20);
	mainWindow.HeightValue:SetPosition(mainWindow.SelList:GetLeft(),mainWindow.HeightCaption:GetTop());
	mainWindow.HeightValue:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleLeft);
	mainWindow.HeightValue:SetFont(Settings.fontFace);
	mainWindow.HeightValue:SetText("20");

	mainWindow.ColorCaption=Turbine.UI.Label();
	mainWindow.ColorCaption:SetParent(mainWindow);
	mainWindow.ColorCaption:SetPosition(5,mainWindow.HeightCaption:GetTop()+mainWindow.HeightCaption:GetHeight()+5);
	mainWindow.ColorCaption:SetSize(mainWindow.SelCaption:GetWidth(),20);
	mainWindow.ColorCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
	mainWindow.ColorCaption:SetFont(Settings.fontFace);
	mainWindow.ColorCaption:SetForeColor(Settings.fontColor);
	mainWindow.ColorCaption:SetText(Resource[language][11]..":");
	mainWindow.ColorValue=ColorPicker();
	mainWindow.ColorValue:SetParent(mainWindow);
	-- the color control has a fixed width=276
	mainWindow.ColorValue:SetPosition(mainWindow.SelList:GetLeft(),mainWindow.ColorCaption:GetTop());
	mainWindow.ColorValue:SetColor(Turbine.UI.Color(0,0,0));

	mainWindow.OpacityCaption=Turbine.UI.Label();
	mainWindow.OpacityCaption:SetParent(mainWindow);
	mainWindow.OpacityCaption:SetPosition(5,mainWindow.ColorCaption:GetTop()+mainWindow.ColorCaption:GetHeight()+5);
	mainWindow.OpacityCaption:SetSize(mainWindow.SelCaption:GetWidth(),20);
	mainWindow.OpacityCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
	mainWindow.OpacityCaption:SetFont(Settings.fontFace);
	mainWindow.OpacityCaption:SetForeColor(Settings.fontColor);
	mainWindow.OpacityCaption:SetText(Resource[language][12]..":");
	mainWindow.OpacityValue=Turbine.UI.Lotro.ScrollBar();
	mainWindow.OpacityValue:SetParent(mainWindow);
	mainWindow.OpacityValue:SetSize(mainWindow.SelList:GetWidth(),10);
	mainWindow.OpacityValue:SetPosition(mainWindow.SelList:GetLeft(),mainWindow.OpacityCaption:GetTop()+5);
	mainWindow.OpacityValue:SetMinimum(0);
	mainWindow.OpacityValue:SetMaximum(100);
	mainWindow.OpacityValue:SetValue(0);

	mainWindow.IconCaption=Turbine.UI.Label();
	mainWindow.IconCaption:SetParent(mainWindow);
	mainWindow.IconCaption:SetPosition(5,mainWindow.OpacityCaption:GetTop()+mainWindow.OpacityCaption:GetHeight()+5);
	mainWindow.IconCaption:SetSize(mainWindow.SelCaption:GetWidth(),20);
	mainWindow.IconCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
	mainWindow.IconCaption:SetFont(Settings.fontFace);
	mainWindow.IconCaption:SetForeColor(Settings.fontColor);
	mainWindow.IconCaption:SetText(Resource[language][13]..":");
	mainWindow.IconQSBack=Turbine.UI.Control();
	mainWindow.IconQSBack:SetParent(mainWindow);
	mainWindow.IconQSBack:SetSize(36,36);
	mainWindow.IconQSBack:SetPosition(mainWindow.SelList:GetLeft(),mainWindow.IconCaption:GetTop());
	mainWindow.IconQSBack:SetBackColor(Settings.trimColor);
	mainWindow.IconQS=Turbine.UI.Lotro.Quickslot();
	mainWindow.IconQS:SetParent(mainWindow.IconQSBack);
	mainWindow.IconQS:SetSize(35,35);
	mainWindow.IconQS:SetPosition(-1,-1);
	mainWindow.IconQS.DragDrop=function(sender,args)
		mainWindow.IconQS.Type=nil;
		mainWindow.IconQS.Data=nil;
		local shortcut = args.DragDropInfo:GetShortcut();
		if ( shortcut ~= nil ) then
			mainWindow.IconQS.Type=shortcut:GetType();
			mainWindow.IconQS.Data=shortcut:GetData();
if Settings.debug then
	Turbine.Shell.WriteLine("IconQS.DragDrop()");
	Turbine.Shell.WriteLine("shortcut:GetType="..tostring(shortcut:GetType()));
	Turbine.Shell.WriteLine("shortcut:GetData="..tostring(shortcut:GetData()));
end
			if shortcut:GetType()==2 then
				mainWindow.IconII:SetItemInfo(shortcut:GetItem():GetItemInfo());
				mainWindow.IconII:SetVisible(true);
				mainWindow.IconQS:SetVisible(false);
			else
				mainWindow.IconII:SetVisible(false);
			end
		end
	end
	mainWindow.IconQS.Type=nil;
	mainWindow.IconQS.Data=nil;

	mainWindow.IconII=Turbine.UI.Lotro.ItemInfoControl()
	mainWindow.IconII:SetParent(mainWindow.IconQSBack);
	mainWindow.IconII:SetSize(34,34);
	mainWindow.IconII:SetPosition(-1,-1);
	mainWindow.IconII:SetItemInfo(nil);
	mainWindow.IconII:SetWantsUpdates(false);
	mainWindow.IconII:SetVisible(false);
	mainWindow.IconII:SetAllowDrop(true);
	mainWindow.IconII.Update=function()
		-- set the icon based on the quickslot shortcut
		local sc=mainWindow.IconQS:GetShortcut()
		if sc:GetType()==2 then
			mainWindow.IconII:SetWantsUpdates(false);
			mainWindow.IconII:SetItemInfo(sc:GetItem():GetItemInfo());
		end
	end
	mainWindow.IconII.DragDrop=function(sender,args)
		mainWindow.IconQS.Type=nil;
		mainWindow.IconQS.Data=nil;
		local shortcut = args.DragDropInfo:GetShortcut();
		if ( shortcut ~= nil ) then
			mainWindow.IconQS.Type=shortcut:GetType();
			mainWindow.IconQS.Data=shortcut:GetData();
			mainWindow.IconQS:SetShortcut(shortcut);
if Settings.debug then
	Turbine.Shell.WriteLine("shortcut:GetType="..tostring(shortcut:GetType()));
	Turbine.Shell.WriteLine("shortcut:GetData="..tostring(shortcut:GetData()));
end
			if shortcut:GetType()==2 then
				mainWindow.IconII:SetItemInfo(shortcut:GetItem():GetItemInfo());
			else
				mainWindow.IconII:SetVisible(false);
				mainWindow.IconQS:SetVisible(true);
			end
		end
	end

	mainWindow.TextCaption=Turbine.UI.Label();
	mainWindow.TextCaption:SetParent(mainWindow);
	mainWindow.TextCaption:SetPosition(5,mainWindow.IconCaption:GetTop()+mainWindow.IconQSBack:GetHeight()+5);
	mainWindow.TextCaption:SetSize(mainWindow.SelCaption:GetWidth(),20);
	mainWindow.TextCaption:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleRight);
	mainWindow.TextCaption:SetFont(Settings.fontFace);
	mainWindow.TextCaption:SetForeColor(Settings.fontColor);
	mainWindow.TextCaption:SetText(Resource[language][14]..":");
	mainWindow.TextBack=Turbine.UI.Control(); -- scrollable text controls have to be in a non-window container for their scrollbars to work now. weird.
	mainWindow.TextBack:SetParent(mainWindow);
	mainWindow.TextBack:SetSize(mainWindow.SelList:GetWidth(),100);
	mainWindow.TextBack:SetPosition(mainWindow.SelList:GetLeft(),mainWindow.TextCaption:GetTop());
	mainWindow.TextValue=Turbine.UI.Lotro.TextBox();
	mainWindow.TextValue:SetParent(mainWindow.TextBack);
	mainWindow.TextValue:SetSize(mainWindow.SelList:GetWidth()-10,100);
	mainWindow.TextValue:SetPosition(0,0);
	mainWindow.TextValue:SetFont(Settings.fontFace);
	mainWindow.TextValue:SetMultiline(true);
	mainWindow.TextVScroll=Turbine.UI.Lotro.ScrollBar();
	mainWindow.TextVScroll:SetParent(mainWindow.TextBack);
	mainWindow.TextVScroll:SetOrientation(Turbine.UI.Orientation.Vertical);
	mainWindow.TextVScroll:SetSize(10,mainWindow.TextValue:GetHeight());
	mainWindow.TextVScroll:SetPosition(mainWindow.TextValue:GetWidth(),0);
	mainWindow.TextVScroll:SetVisible(false);
	mainWindow.TextValue:SetVerticalScrollBar(mainWindow.TextVScroll);

	mainWindow.SelList:AddItem(Resource[language][5],0)

	mainWindow.SaveButton=Turbine.UI.Lotro.Button();
	mainWindow.SaveButton:SetParent(mainWindow);
	mainWindow.SaveButton:SetSize(100,20);
	mainWindow.SaveButton:SetFont(Settings.fontFace);
	mainWindow.SaveButton:SetText(Resource[language][15]);
	mainWindow.SaveButton:SetPosition(25,mainWindow.TextBack:GetTop()+mainWindow.TextBack:GetHeight()+15);
	mainWindow.SaveButton.Click=function()
		local index=mainWindow.SelList:GetValue();
if Settings.debug then
Turbine.Shell.WriteLine("saving tip:"..tostring(index))
end
		local hasError=false;
		mainWindow.NameCaption:SetForeColor(Settings.fontColor);
		mainWindow.LeftCaption:SetForeColor(Settings.fontColor);
		mainWindow.TopCaption:SetForeColor(Settings.fontColor);
		mainWindow.WidthCaption:SetForeColor(Settings.fontColor);
		mainWindow.HeightCaption:SetForeColor(Settings.fontColor);
		local value=string.trim(mainWindow.NameValue:GetText());
		if value == nil or value=="" then
			mainWindow.NameCaption:SetForeColor(Settings.errorColor);
			hasError=true;
		end
		value=tonumber(mainWindow.LeftValue:GetText());
		if value==nil then
			mainWindow.LeftCaption:SetForeColor(Settings.errorColor);
			hasError=true;
		else
			value=math.floor(value);
			if value<1 or value>Turbine.UI.Display:GetWidth() then
				mainWindow.LeftCaption:SetForeColor(Settings.errorColor);
				hasError=true;
			end
		end
		value=tonumber(mainWindow.TopValue:GetText());
		if value==nil then
			mainWindow.TopCaption:SetForeColor(Settings.errorColor);
			hasError=true;
		else
			value=math.floor(value);
			if value<1 or value>Turbine.UI.Display:GetHeight() then
				mainWindow.TopCaption:SetForeColor(Settings.errorColor);
				hasError=true;
			end
		end
		value=tonumber(mainWindow.WidthValue:GetText());
		if value==nil then
			mainWindow.WidthCaption:SetForeColor(Settings.errorColor);
			hasError=true;
		else
			value=math.floor(value);
			if value<1 or value>Turbine.UI.Display:GetWidth() then
				mainWindow.WidthCaption:SetForeColor(Settings.errorColor);
				hasError=true;
			end
		end
		value=tonumber(mainWindow.HeightValue:GetText());
		if value==nil then
			mainWindow.HeightCaption:SetForeColor(Settings.errorColor);
			hasError=true;
		else
			value=math.floor(value);
			if value<1 or value>Turbine.UI.Display:GetHeight() then
				mainWindow.HeightCaption:SetForeColor(Settings.errorColor);
				hasError=true;
			end
		end
		if hasError then
		else
			local newName=string.trim(mainWindow.NameValue:GetText());
			if index==0 then
				index=#customTips+1;
				customTips[index]={};
				hotSpots[index]=HotSpot();
				hotSpots[index].Index=index;
				mainWindow.SelList:AddItem(newName,index);
			else
				mainWindow.SelList.ListData:GetItem(index+1):SetText(newName);
				mainWindow.SelList.CurrentValue:SetText(newName);
			end
			customTips[index].Name=newName;
			customTips[index].Left=tonumber(mainWindow.LeftValue:GetText());
			customTips[index].Top=tonumber(mainWindow.TopValue:GetText());
			customTips[index].Width=tonumber(mainWindow.WidthValue:GetText());
			customTips[index].Height=tonumber(mainWindow.HeightValue:GetText());
			customTips[index].Color=mainWindow.ColorValue:GetColor();
			customTips[index].Opacity=mainWindow.OpacityValue:GetValue()/100;
			customTips[index].IconType=mainWindow.IconQS.Type;
			customTips[index].IconData=mainWindow.IconQS.Data;
			customTips[index].Text=string.trim(mainWindow.TextValue:GetText());

			hotSpots[index]:SetPosition(customTips[index].Left,customTips[index].Top);
			hotSpots[index]:SetSize(customTips[index].Width,customTips[index].Height);
			hotSpots[index]:SetBackColor(customTips[index].Color);
			hotSpots[index]:SetOpacity(customTips[index].Opacity);
			mainWindow.SelList:SetSelectedIndex(index+1);
		end
	end
	
	mainWindow.DeleteButton=Turbine.UI.Lotro.Button();
	mainWindow.DeleteButton:SetParent(mainWindow);
	mainWindow.DeleteButton:SetSize(100,20);
	mainWindow.DeleteButton:SetFont(Settings.fontFace);
	mainWindow.DeleteButton:SetText(Resource[language][16]);
	mainWindow.DeleteButton:SetPosition(mainWindow:GetWidth()-mainWindow.DeleteButton:GetWidth()-mainWindow.SaveButton:GetLeft(),mainWindow.SaveButton:GetTop());
	mainWindow.DeleteButton:SetEnabled(false);
	mainWindow.DeleteButton.Click=function()
		local selIndex=mainWindow.SelList:GetSelectedIndex();
		if selIndex>0 then
			local index=mainWindow.SelList:GetValue();
			table.remove(customTips,index);
			hotSpots[index]:SetVisible(false);
			table.remove(hotSpots,index);
			local k;
			for k=selIndex, mainWindow.SelList.ListData:GetItemCount() do
				mainWindow.SelList.ListData:GetItem(k).DataValue=mainWindow.SelList.ListData:GetItem(k).DataValue-1;
			end
			mainWindow.SelList:RemoveItemAt(selIndex);
		end
	end
mainWindow.Update=function( sender, args )
	if (Plugins.CustomTips ~= nil) and (not mainWindow.Loaded) then
		mainWindow.Loaded=true;
		Turbine.Shell.WriteLine("Custom Tips "..Plugins.CustomTips:GetVersion()..Resource[language][3]);
		Plugins.CustomTips.Unload = function(mainWindow,sender,args)
			UnloadPlugin();
		end
	end
end

if Settings.debug then
	debugWindow=DebugWindow();
	debugWindow:SetVisible(true);
end

highlightColor=Settings.trimColor;
highlightColor.R=highlightColor.R*1.1;
highlightColor.G=highlightColor.G*1.1;
highlightColor.B=highlightColor.B*1.1;
if highlightColor.R>1 then highlightColor.R=1 end
if highlightColor.G>1 then highlightColor.G=1 end
if highlightColor.B>1 then highlightColor.B=1 end
toolTipWindow=Turbine.UI.Window();
toolTipWindow:SetPosition(450,300);
toolTipWindow:SetSize(300,200);
toolTipWindow.Clicked=false;
toolTipWindow:SetVisible(false)
toolTipWindow.Border=Turbine.UI.Control();
toolTipWindow.Border:SetParent(toolTipWindow);
toolTipWindow.Border:SetSize(toolTipWindow:GetSize());
toolTipWindow.Border:SetBackColor(trimColor)
toolTipWindow.Highlight1=Turbine.UI.Control();
toolTipWindow.Highlight1:SetParent(toolTipWindow.Border);
toolTipWindow.Highlight1:SetSize(toolTipWindow:GetWidth()-1,1);
toolTipWindow.Highlight1:SetPosition(0,0)
toolTipWindow.Highlight1:SetBackColor(highlightColor);
toolTipWindow.Highlight2=Turbine.UI.Control();
toolTipWindow.Highlight2:SetParent(toolTipWindow);
toolTipWindow.Highlight2:SetSize(1,toolTipWindow:GetHeight()-1)
toolTipWindow.Highlight2:SetPosition(0,0);
toolTipWindow.Highlight2:SetBackColor(highlightColor);
toolTipWindow.Highlight3=Turbine.UI.Control();
toolTipWindow.Highlight3:SetParent(toolTipWindow);
toolTipWindow.Highlight3:SetSize(toolTipWindow:GetWidth()-2,2);
toolTipWindow.Highlight3:SetPosition(1,toolTipWindow:GetHeight()-3);
toolTipWindow.Highlight3:SetBackColor(highlightColor);
toolTipWindow.Highlight4=Turbine.UI.Control();
toolTipWindow.Highlight4:SetParent(toolTipWindow);
toolTipWindow.Highlight4:SetSize(2,toolTipWindow:GetHeight()-2)
toolTipWindow.Highlight4:SetPosition(toolTipWindow:GetWidth()-3,1);
toolTipWindow.Highlight4:SetBackColor(highlightColor);
toolTipWindow.Background=Turbine.UI.Control();
toolTipWindow.Background:SetParent(toolTipWindow);
toolTipWindow.Background:SetBackColor(backColor);
toolTipWindow.Background:SetPosition(2,2);
toolTipWindow.Background:SetSize(toolTipWindow:GetWidth()-4,toolTipWindow:GetHeight()-4);
toolTipWindow.Icon=Turbine.UI.Control();
toolTipWindow.Icon:SetParent(toolTipWindow.Background);
toolTipWindow.IconQS=Turbine.UI.Lotro.Quickslot();
toolTipWindow.IconQS:SetParent(toolTipWindow.Icon);
toolTipWindow.IconQS:SetVisible(false);
toolTipWindow.IconQS:SetMouseVisible(false);
toolTipWindow.IconII=Turbine.UI.Lotro.ItemInfoControl();
toolTipWindow.IconII:SetParent(toolTipWindow.Icon);
toolTipWindow.IconII:SetVisible(false);
toolTipWindow.IconII:SetMouseVisible(false);
toolTipWindow.Title=Turbine.UI.Label();
toolTipWindow.Title:SetParent(toolTipWindow.Background);
toolTipWindow.Title:SetBackColor(backColor);
toolTipWindow.Title:SetFont(Turbine.UI.Lotro.Font.Verdana20);
toolTipWindow.Title:SetMarkupEnabled(true);
toolTipWindow.Title:SetHeight(40);
toolTipWindow.Title:SetTop(4);
toolTipWindow.Title:SetTextAlignment(Turbine.UI.ContentAlignment.MiddleCenter);
toolTipWindow.Description=Turbine.UI.Label();
toolTipWindow.Description:SetParent(toolTipWindow.Background);
toolTipWindow.Description:SetBackColor(backColor);
toolTipWindow.Description:SetPosition(5,49);
toolTipWindow.Description:SetSize(toolTipWindow.Background:GetWidth()-10,10);
toolTipWindow.Description.VScroll=Turbine.UI.Lotro.ScrollBar();
toolTipWindow.Description.VScroll:SetParent(toolTipWindow.Description);
toolTipWindow.Description.VScroll:SetSize(10,100);
toolTipWindow.Description.VScroll:SetPosition(-10,0);
toolTipWindow.Description.VScroll:SetVisible(false);
toolTipWindow.Description:SetVerticalScrollBar(toolTipWindow.Description.VScroll);
toolTipWindow.Description:SetMarkupEnabled(true);
toolTipWindow.CloseButton=Turbine.UI.Control();
toolTipWindow.CloseButton:SetParent(toolTipWindow.Background);
toolTipWindow.CloseButton:SetSize(16,16);
toolTipWindow.CloseButton:SetPosition(toolTipWindow:GetWidth()-20,0);
toolTipWindow.CloseButton:SetBackground(0x41000196);
toolTipWindow.CloseButton:SetBlendMode(Turbine.UI.BlendMode.Overlay);
toolTipWindow.CloseButton.MouseEnter=function()
	toolTipWindow.CloseButton:SetBackground(0x41000198);
end
toolTipWindow.CloseButton.MouseLeave=function()
	toolTipWindow.CloseButton:SetBackground(0x41000196);
end
toolTipWindow.CloseButton.MouseDown=function()
	toolTipWindow.CloseButton:SetBackground(0x41000197);
end
toolTipWindow.CloseButton.MouseClick=function()
	toolTipWindow:Close();
end
toolTipWindow.Show=function(sender,tipIndex,X,Y)
	if tipIndex~=nil then
		tipIndex=math.floor(tonumber(tipIndex));
		if tipIndex>0 and tipIndex<=#customTips then
			
			if customTips[tipIndex].IconType==nil then
				-- hide icon
				toolTipWindow.Icon:SetVisible(false)
				toolTipWindow.Title:SetLeft(7)
				toolTipWindow.Title:SetWidth(toolTipWindow:GetWidth()-32)
			else
				toolTipWindow.Icon:SetVisible(true)
				toolTipWindow.Title:SetLeft(46)
				toolTipWindow.Title:SetWidth(toolTipWindow:GetWidth()-78)
				local sc=Turbine.UI.Lotro.Shortcut(customTips[tipIndex].IconType,0);
				sc:SetData(customTips[tipIndex].IconData);
				if customTips[tipIndex].IconType==2 then
					-- show iteminfo
					toolTipWindow.IconII:SetVisible(true);
					toolTipWindow.IconQS:SetVisible(false);
					toolTipWindow.IconII:SetItemInfo(sc:GetItem():GetItemInfo());
				else
					-- show quickslot
					toolTipWindow.IconII:SetVisible(false);
					toolTipWindow.IconQS:SetVisible(true);
					toolTipWindow.IconQS:SetShortcut(sc);
				end
			end
			toolTipWindow.Title:SetText(customTips[tipIndex].Name);
			toolTipWindow.Description:SetHeight(10);
			toolTipWindow.Description:SetText(customTips[tipIndex].Text);
			while toolTipWindow.Description.VScroll:IsVisible() do
				toolTipWindow.Description:SetHeight(toolTipWindow.Description:GetHeight()+10);
			end
			toolTipWindow.Background:SetHeight(toolTipWindow.Description:GetHeight()+54);
			toolTipWindow.Border:SetHeight(toolTipWindow.Background:GetHeight()+4)
			toolTipWindow:SetHeight(toolTipWindow.Border:GetHeight());
			toolTipWindow.Highlight2:SetHeight(toolTipWindow:GetHeight()-1);
			toolTipWindow.Highlight3:SetTop(toolTipWindow:GetHeight()-3);
			toolTipWindow.Highlight4:SetHeight(toolTipWindow:GetHeight()-2)
			toolTipWindow:SetVisible(true)
		end
	end
end

-- the tooltip hotspot class
HotSpot=class(Turbine.UI.Window)
function HotSpot:Constructor()
	Turbine.UI.Window.Constructor(self)
	self.MouseHover=function(sender,args)
		toolTipWindow.Clicked=false;
		toolTipWindow:Show(self.Index,args.X,args.Y);
	end
	self.MouseClick=function(sender,args)
		toolTipWindow.Clicked=true;
		toolTipWindow:Show(self.Index,args.X,args.Y);
	end
	self.MouseLeave=function()
		if not toolTipWindow.Clicked then
			toolTipWindow:SetVisible(false);
		end
	end
	self:SetVisible(true);
end

hotSpots={};
for k,v in ipairs(customTips) do
	mainWindow.SelList:AddItem(v.Name,k)
	hotSpots[k]=HotSpot();
	hotSpots[k].Index=k;
	hotSpots[k]:SetSize(v.Width,v.Height);
	hotSpots[k]:SetPosition(v.Left,v.Top);
	hotSpots[k]:SetBackColor(v.Color);
	hotSpots[k]:SetOpacity(v.Opacity);
end

layoutWindow=Turbine.UI.Window();
layoutWindow:SetBackColor(Turbine.UI.Color(1,0,0));
layoutWindow.OuterBorder=Turbine.UI.Control();
layoutWindow.OuterBorder:SetParent(layoutWindow);
layoutWindow.OuterBorder:SetBackColor(Turbine.UI.Color(1,1,1));
layoutWindow.OuterBorder:SetPosition(1,1);
layoutWindow.OuterBorder:SetMouseVisible(false);
layoutWindow.InnerBorder=Turbine.UI.Control();
layoutWindow.InnerBorder:SetParent(layoutWindow.OuterBorder);
layoutWindow.InnerBorder:SetBackColor(Turbine.UI.Color(1,0,0));
layoutWindow.InnerBorder:SetPosition(1,1);
layoutWindow.InnerBorder:SetMouseVisible(false);
layoutWindow.Center=Turbine.UI.Control();
layoutWindow.Center:SetParent(layoutWindow.InnerBorder);
layoutWindow.Center:SetBackColor(Turbine.UI.Color(0,1,1,1));
layoutWindow.Center:SetPosition(1,1);
layoutWindow.moving=false;
layoutWindow.sizingLeft=false;
layoutWindow.sizingRight=false;
layoutWindow.sizingTop=false;
layoutWindow.sizingBottom=false;
layoutWindow.Display=function()
	local val1,val2=mainWindow.LeftValue:GetText(),mainWindow.TopValue:GetText();
	val1=tonumber(val1);
	val2=tonumber(val2);
	if val1==nil then val1=0 end
	if val2==nil then val2=0 end
	if val1<0 then val1=0 end
	if val2<0 then val2=0 end
	if val1>Turbine.UI.Display:GetWidth()-1 then val1=Turbine.UI.display:GetWidth()-1 end
	if val2>Turbine.UI.Display:GetHeight()-1 then val2=Turbine.UI.display:GetHeight()-1 end
	layoutWindow:SetPosition(val1,val2);

	val1,val2=mainWindow.WidthValue:GetText(),mainWindow.HeightValue:GetText();
	val1=tonumber(val1);
	val2=tonumber(val2);
	if val1==nil then val1=0 end
	if val2==nil then val2=0 end
	if val1<3 then val1=3 end
	if val2<3 then val2=3 end
	if val1>Turbine.UI.Display:GetWidth() then val1=Turbine.UI.display:GetWidth() end
	if val2>Turbine.UI.Display:GetHeight() then val2=Turbine.UI.display:GetHeight() end
	layoutWindow:SetSize(val1,val2);

	layoutWindow.OuterBorder:SetSize(val1-2,val2-2);
	layoutWindow.InnerBorder:SetSize(val1-4,val2-4);
	layoutWindow.Center:SetSize(val1-6,val2-6);
end
layoutWindow:Display();
layoutWindow.Center.MouseDown=function(sender,args)
	layoutWindow.x=args.X;
	layoutWindow.y=args.Y;
	layoutWindow.moving=true;
end
layoutWindow.Center.MouseUp=function(sender,args)
	layoutWindow.moving=false;
end
layoutWindow.Center.MouseMove=function(sender,args)
	if layoutWindow.moving then
		local newx=layoutWindow:GetLeft()-(layoutWindow.x-args.X);
		local newy=layoutWindow:GetTop()-(layoutWindow.y-args.Y);
		if newx<0 then newx=0 end;
		if newx+layoutWindow:GetWidth()>Turbine.UI.Display:GetWidth() then newx=Turbine.UI.Display:GetWidth()-layoutWindow:GetWidth() end
		if newy<0 then newy=0 end;
		if newy+layoutWindow:GetHeight()>Turbine.UI.Display:GetHeight() then newy=Turbine.UI.Display:GetHeight()-layoutWindow:GetHeight() end
		layoutWindow:SetPosition(newx,newy);
		mainWindow.LeftValue:SetText(tostring(newx));
		mainWindow.TopValue:SetText(tostring(newy));
	end
end
layoutWindow.MouseDown=function(sender,args)
	layoutWindow.x=args.X;
	layoutWindow.y=args.Y;
	if args.X<4 then layoutWindow.sizingLeft=true end
	if args.X>layoutWindow:GetWidth()-4 then layoutWindow.sizingRight=true end
	if args.Y<4 then layoutWindow.sizingTop=true end
	if args.Y>layoutWindow:GetHeight()-4 then layoutWindow.sizingBottom=true end	
end
layoutWindow.MouseUp=function()
	layoutWindow.sizingLeft=false;
	layoutWindow.sizingRight=false;
	layoutWindow.sizingTop=false;
	layoutWindow.sizingBottom=false;
end
layoutWindow.MouseMove=function(sender,args)
	if layoutWindow.sizingLeft then
		local deltax=layoutWindow.x-args.X;
		if layoutWindow:GetWidth()+deltax<6 then
			deltax=6-layoutWindow:GetWidth()
		end
		if layoutWindow:GetWidth()+deltax>Turbine.UI.Display:GetWidth() then
			deltax=Turbine.UI.Display:GetWidth()-layoutWindow:GetWidth()
		end
		local newLeft=layoutWindow:GetLeft()-deltax;
		if newLeft<0 then
			newLeft=0;
			deltax=0-layoutWindow:GetLeft();
		end
		if newLeft+layoutWindow:GetWidth()>Turbine.UI.Display:GetWidth() then
			newLeft=Turbine.UI.Display:GetWidth()-layoutWindow:GetWidth();
			deltax=Turbine.UI.Display:GetWidth()-layoutWindow:GetWidth()-layoutWindow:GetLeft();
		end
		layoutWindow:SetLeft(newLeft);
		layoutWindow:SetWidth(layoutWindow:GetWidth()+deltax);
		layoutWindow.OuterBorder:SetWidth(layoutWindow:GetWidth()-2);
		layoutWindow.InnerBorder:SetWidth(layoutWindow.OuterBorder:GetWidth()-2);
		layoutWindow.Center:SetWidth(layoutWindow.InnerBorder:GetWidth()-2);
	end
	if layoutWindow.sizingTop then
		local deltay=layoutWindow.y-args.Y;
		if layoutWindow:GetHeight()+deltay<6 then
			deltay=6-layoutWindow:GetHeight()
		end
		if layoutWindow:GetHeight()+deltay>Turbine.UI.Display:GetHeight() then
			deltay=Turbine.UI.Display:GetHeight()-layoutWindow:GetHeight()
		end
		local newTop=layoutWindow:GetTop()-deltay;
		if newTop<0 then
			newTop=0;
			deltay=0-layoutWindow:GetTop();
		end
		if newTop+layoutWindow:GetHeight()>Turbine.UI.Display:GetHeight() then
			newTop=Turbine.UI.Display:GetHeight()-layoutWindow:GetHeight();
			deltay=Turbine.UI.Display:GetHeight()-layoutWindow:GetHeight()-layoutWindow:GetTop();
		end
		layoutWindow:SetTop(newTop);
		layoutWindow:SetHeight(layoutWindow:GetHeight()+deltay);
		layoutWindow.OuterBorder:SetHeight(layoutWindow:GetHeight()-2);
		layoutWindow.InnerBorder:SetHeight(layoutWindow.OuterBorder:GetHeight()-2);
		layoutWindow.Center:SetHeight(layoutWindow.InnerBorder:GetHeight()-2);
	end
	if layoutWindow.sizingRight then
		local deltax=args.X-layoutWindow.x;
		if layoutWindow:GetWidth()+deltax<6 then
			deltax=6-layoutWindow:GetWidth()
		end
		if layoutWindow:GetLeft()+layoutWindow:GetWidth()+deltax>Turbine.UI.Display:GetWidth() then
			deltax=Turbine.UI.Display:GetWidth()-layoutWindow:GetWidth()-layoutWindow:GetLeft()
		end
		layoutWindow:SetWidth(layoutWindow:GetWidth()+deltax);
		layoutWindow.OuterBorder:SetWidth(layoutWindow:GetWidth()-2);
		layoutWindow.InnerBorder:SetWidth(layoutWindow.OuterBorder:GetWidth()-2);
		layoutWindow.Center:SetWidth(layoutWindow.InnerBorder:GetWidth()-2);
		layoutWindow.x=args.X;
	end
	if layoutWindow.sizingBottom then
		local deltay=args.Y-layoutWindow.y;
		if layoutWindow:GetHeight()+deltay<6 then
			deltay=6-layoutWindow:GetHeight()
		end
		if layoutWindow:GetTop()+layoutWindow:GetHeight()+deltay>Turbine.UI.Display:GetHeight() then
			deltay=Turbine.UI.Display:GetHeight()-layoutWindow:GetHeight()-layoutWindow:GetTop()
		end
		layoutWindow:SetHeight(layoutWindow:GetHeight()+deltay);
		layoutWindow.OuterBorder:SetHeight(layoutWindow:GetHeight()-2);
		layoutWindow.InnerBorder:SetHeight(layoutWindow.OuterBorder:GetHeight()-2);
		layoutWindow.Center:SetHeight(layoutWindow.InnerBorder:GetHeight()-2);
		layoutWindow.y=args.Y;
	end
	mainWindow.LeftValue:SetText(tostring(layoutWindow:GetLeft()));
	mainWindow.TopValue:SetText(tostring(layoutWindow:GetTop()));
	mainWindow.WidthValue:SetText(tostring(layoutWindow:GetWidth()));
	mainWindow.HeightValue:SetText(tostring(layoutWindow:GetHeight()));
end
mainWindow.VisibleChanged=function()
	layoutWindow:SetVisible(mainWindow:IsVisible());
end
